/*****************************************************************************
         (c) Copyright Effice Mrnki Iroda Kft. 1999.

Ez az llomny kizrolag az Effice Kft. (2083 Solymr Toldi u. 21.) tulajdona,
kizrolag csak az Effice Kft. engedlyvel, az engedlyben meghatrozott clra
s mdon hasznlhat fel. A program ingyenesen trtnt tadsra, mkdsrt 
az Effice Kft. semmilyen felelssget nem vllal.

Demo program about using ifsc functions.
*****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "ifsapi.h"
#include "ifsdefs.h"
#include "ifsutil.h"

/* static char Id[] = ""; */

static char * print_message(char * buf, char * msg, int len);
void usage( void);

/*+++++++++++++++++++++++++++++++++++++++++
Prints usage information.
+++++++++++++++++++++++++++++++++++++++++*/
void usage( void){
  fprintf(stderr, "Usage [-hhost] [-sservice] [-uuser] [-ppwd]\n");
  fprintf(stderr, "  -hhost     host name of server\n");
  fprintf(stderr, "  -sservice  service name of server\n");
  fprintf(stderr, "  -uuser     user name\n");
  fprintf(stderr, "  -pwd       password\n");
  fprintf(stderr, "If no host, service, user, pwd defined then \n");
  fprintf(stderr, "  value of IFSHOST, IFSSERVICE, IFSUSER, IFSPWD environment used\n");
  fprintf(stderr, "\n");
  exit(1);    
}

/*+++++++++++++++++++++++++++++++++++++++++
Print a message.

char * print_message Points to the message.

char * buf Pointer to the print buffer(output).

char * msg Pointer to message (input).

int len Length of the input.
+++++++++++++++++++++++++++++++++++++++++*/
static char * print_message(char * buf, char * msg, int len)
{
  int i;
  for (i = 0; i < len; i++){
    if (msg[i]) 
      buf[i] = msg[i];
    else
      buf[i] = '|'; 
  }
  buf[i]='\0';
  return buf;
}

typedef struct {
  int     ordId;
  int     ordNo;
  char    trdAccId[IFS_IDS_LEN];
  char    buySell;
  char    orderMethodSet[IFS_ORDERMETHOD_LEN];
  char    boardId[IFS_BOARDID_LEN];
  char    secId[IFS_IDS_LEN];
  int     price;
  int     yield;
  int     quantity;
  int     hidden;
  char    brokerRef[IFS_BROKERREF_LEN];
  int     expDate;
  int     expTime;
  int     triggerPrice;
  int     settleDate;
  int     minFillQty;
  char    opCode;
  char    popCode;
  char    userId[IFS_IDS_LEN];
  char    firmId[IFS_IDS_LEN];
  int     orderDate;
  char    status;
  char    type;
  char    errmsg[IFS_MSG_LEN];
  char    internalRef[IFS_BROKERREF_LEN];
} orderentry;

/*+++++++++++++++++++++++++++++++++++++++++
Queries secboard and orderentry table. Send an orderentry.
int main Tradition.

int argc Number of command line parameters.

char *argv[] Points to an array that contains the command line parameters.
+++++++++++++++++++++++++++++++++++++++++*/
int main (int argc, char *argv[])
{
  int s, i, ret=0, len, j, seqno, tradeid, pid, l, orderid, mmts_type;
  orderentry data;
  char * recvb;
  char ptr_buf[0x8000];
  char * host = NULL;
  char * service = NULL;
  char * user = NULL;
  char * pwd = NULL;
  char buf[1024];
  IFSC_HDL handle;

  host = getenv("IFSHOST");
  service = getenv("IFSSERVICE");
  user = getenv("IFSUSER");
  pwd = getenv("IFSPWD");


  for (i = 1; i < argc; i++){
    if (argv[i][0] == '-'){
      switch(argv[i][1]){
        case 'h':
          host = argv[i]+2;
          break;
        case 's':
          service = argv[i]+2;
          break;
        case 'u':
          user = argv[i]+2;
          break;
        case 'p':
          pwd = argv[i]+2;
          break;
      }
    } else {
        usage();
    }  
  }

  if (!host) {
    fprintf(stderr, "The host is not defined!\n");
    usage();
  }

  if (!service) {
    fprintf(stderr, "The service is not defined!\n");
    usage();
  }

  if (!user) {
    fprintf(stderr, "The user is not defined!\n");
    usage();
  }

  if (!pwd) {
    fprintf(stderr, "The pwd is not defined!\n");
    usage();
  }

  if (ifsc_create(&handle, host, service)) {
    fprintf(stderr, "Error at creation: %s\n", ifsc_get_last_errmsg(handle));
    exit(1);
  }

  if (ifsc_connect(handle, user, pwd, &tradeid, &pid, &mmts_type)) {
    fprintf(stderr, "Error at connection: %s\n", ifsc_get_last_errmsg(handle));
    exit(1);
  }

  /* queries records of secboard table while new change arriving */
  /* and prints that into stdout */
  while ((s = ifsc_get_next_record(handle, &recvb, &len, IFS_T_SECBOARD, &seqno)) == 0) {
    fprintf(stderr, "ifsc_get_next_record IFS_T_SECBOARD : status: %d, len %d, seqno: %d\n\n",s, len, seqno); 
    fprintf(stdout, "%s\n", print_message(ptr_buf, recvb, len));
    fflush(stdout);
  }
  
  /* prepare an orderentry record */
  len = ifs_set_ids(buf, "0046-0000001");
  len += ifs_set_char(buf + len, 'B');
  len += ifs_set_string(buf + len, IFS_ORDERMETHOD_LEN, "01000000000000000100000010000000");
  len += ifs_set_string(buf + len, IFS_BOARDID_LEN, "FR");
  len += ifs_set_ids(buf + len, "MOL     ");
  len += ifs_set_int(buf + len, 1000);
  len += ifs_set_int(buf + len, 0);
  len += ifs_set_int(buf + len, 1);
  len += ifs_set_int(buf + len, 0);
  len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, "demo");
  len += ifs_set_int(buf + len, 20000202);
  len += ifs_set_int(buf + len, 0);
  len += ifs_set_int(buf + len, 0);
  len += ifs_set_int(buf + len, 20000203);
  len += ifs_set_int(buf + len, 0);
  len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, "bos ref");

  /* sending an orderentry record */
  if ((s = ifsc_orderentry(handle, IFS_ACTION_ORDER_ADD, buf, len, &orderid)) == 0) {
    fprintf (stderr, " order_add (orderid %d) sent\n", orderid);
  } 
  else
    fprintf (stderr, " error at order_add - %s\n", ifsc_get_last_errmsg(handle));

  /* queries records of orderentry table while new change arriving */
  /* last orderentry is the previously sent one */
  /* decode ordernetry fields into stuct */
  /* prints records into stdout */
  while ((s = ifsc_get_next_record(handle, &recvb, &len, IFS_T_ORDERENTRY, &seqno)) == 0) {

    fprintf(stderr, "ifsc_get_next_record IFS_T_ORDERENTRY: status: %d, len %d, seqno: %d\n\n",s, len, seqno); 

    l = ifs_get_int(&(data.ordId), recvb);
    l += ifs_get_int(&(data.ordNo), recvb + l);
    l += ifs_get_ids(data.trdAccId, IFS_IDS_LEN, recvb + l);
    l += ifs_get_char(&(data.buySell), recvb + l);
    l += ifs_get_string(data.orderMethodSet, IFS_ORDERMETHOD_LEN, recvb + l, IFS_ORDERMETHOD_LEN);
    l += ifs_get_string(data.boardId, IFS_BOARDID_LEN, recvb + l, IFS_BOARDID_LEN);
    l += ifs_get_ids(data.secId, IFS_IDS_LEN, recvb + l);
    l += ifs_get_int(&(data.price), recvb + l);
    l += ifs_get_int(&(data.yield), recvb + l);
    l += ifs_get_int(&(data.quantity), recvb + l);
    l += ifs_get_int(&(data.hidden), recvb + l);
    l += ifs_get_string(data.brokerRef, IFS_BROKERREF_LEN, recvb + l, IFS_BROKERREF_LEN);
    l += ifs_get_int(&(data.expDate), recvb + l);
    l += ifs_get_int(&(data.expTime), recvb + l);
    l += ifs_get_int(&(data.triggerPrice), recvb + l);
    l += ifs_get_int(&(data.settleDate), recvb + l);
    l += ifs_get_int(&(data.minFillQty), recvb + l);
    l += ifs_get_char(&(data.opCode), recvb + l);
    l += ifs_get_char(&(data.popCode), recvb + l);
    l += ifs_get_ids(data.userId, IFS_IDS_LEN, recvb + l);
    l += ifs_get_ids(data.firmId, IFS_IDS_LEN, recvb + l);
    l += ifs_get_int(&(data.orderDate), recvb + l);
    l += ifs_get_char(&(data.status), recvb + l);
    l += ifs_get_char(&(data.type), recvb + l);
    l += ifs_get_string(data.errmsg, IFS_MSG_LEN, recvb + l, IFS_MSG_LEN);
    l += ifs_get_string(data.internalRef, IFS_BROKERREF_LEN, recvb + l, IFS_BROKERREF_LEN);
    
    fprintf(stdout, "%d|%d|%s|%c|%s|%s|%s|%d|%d|%d|%d|%s|%d|%d|%d|%d|%d|%c|%c|%s|%s|%d|%c|%c|%s|%s\n",
  		data.ordId,
  		data.ordNo,
  		data.trdAccId,
  		data.buySell,
  		data.orderMethodSet,
  		data.boardId,
  		data.secId,
  		data.price,
  		data.yield,
  		data.quantity,
  		data.hidden,
  		data.brokerRef,
  		data.expDate,
  		data.expTime,
  		data.triggerPrice,
  		data.settleDate,
  		data.minFillQty,
		data.opCode,
		data.popCode,
  		data.userId,
  		data.firmId,
  		data.orderDate,
  		data.status,
  		data.type,
  		data.errmsg,
		data.internalRef
  	 );

    fprintf(stdout, "%s\n", print_message(ptr_buf, recvb, len));
    fflush(stdout);
  }

  if (ifsc_disconnect(handle)) {
    fprintf(stderr, "Error at disconnect: %s\n", ifsc_get_last_errmsg(handle));
    exit(1);
  }
  return 0;
}
